import abc
import six
from azmessaging.models import SMSNotificationType

@six.add_metaclass(abc.ABCMeta)
class SMSApi:

    def __init__(self, **kwargs):
        pass

    @abc.abstractmethod
    def send(self, phone_number: str, message: str, sms_type: SMSNotificationType):
        pass

    @abc.abstractmethod
    def bulk_send(self, phone_numbers: [str], message: str, sms_type: SMSNotificationType):
        pass



from twilio.rest import Client

from .smsabstract import SMSApi
from ..models import SMSNotificationType


class SMSAPITwilio(SMSApi):

    def __init__(self, account_sid, auth_token, sender, **kwargs):
        super(SMSAPITwilio, self).__init__(**kwargs)
        self.api = Client(
            account_sid,
            auth_token,
        )
        self._sender = sender

    def bulk_send(self, phone_numbers: [str], message: str, sms_type: SMSNotificationType):
        # TODO: handle bulk options
        for phone_number in phone_numbers:
            self.send(phone_number, message, sms_type)

    def send(self, phone_number: str, message: str, sms_type: SMSNotificationType):
        response = self.api.messages.create(
            body=message,
            from_=self._sender,
            to=phone_number
        )
        # TODO: handle response




from azmessaging import default_settings as settings
from .bases import Reader
from .smsconfig import SMSConfig


class DefaultReader(Reader):

    def get_sms_config(self, identifier) -> SMSConfig:
        return SMSConfig(
            default=settings.SMS_CONFIG['DEFAULT_SERVICE_PROVIDER'],
            priorities=settings.SMS_CONFIG['PRIORITY_SERVICE_PROVIDER'],
            service_providers=settings.SMS_CONFIG['SERVICE_PROVIDER'],
            white_list_countries=settings.SMS_CONFIG['WHITE_LIST'],
            black_list_countries=settings.SMS_CONFIG['BLACK_LIST'],
        )



"""Default settings for messaging."""

from django.conf import settings
import django

from .exceptions import AZSettingDoesNotExist
from .utils import import_class

if django.__version__ >= '3.0':
    from django.db import models

    TEXT_CHOICES = models.TextChoices
else:
    from .models.enum_django import TextChoices

    TEXT_CHOICES = TextChoices

MESSAGING = getattr(settings, 'AZ_MESSAGING', {})

SETTING_VALUE_READER_CLASS = getattr(MESSAGING, 'SETTING_VALUE_READER_CLASS', 'azmessaging.readers.DefaultReader')

CHANNEL_CLASS = getattr(
    MESSAGING,
    'CLASS',
    {
        'SMS': 'azmessaging.channels.SMSNotificationChannel',
    }
)

"""
READER
Load in app config after application loaded
"""
READER = None

"""
SMS
"""
SMS_CONFIG = MESSAGING.get('SMS', {})
if len(SMS_CONFIG) != 0:
    SMS_CONFIG['WHITE_LIST'] = SMS_CONFIG.get('WHITE_LIST', '__all__')
    if not SMS_CONFIG.get('DEFAULT_SERVICE_PROVIDER', None) or \
            len(SMS_CONFIG.get('SERVICE_PROVIDER', {}).get(SMS_CONFIG['DEFAULT_SERVICE_PROVIDER'], {})) == 0:
        raise AZSettingDoesNotExist('SMS configuration: please check `DEFAULT_SERVICE_PROVIDER` and `SERVICE_PROVIDER`')

    if len(SMS_CONFIG.get('PRIORITY_SERVICE_PROVIDER', [])) == 0 or \
            len(list(filter(lambda x: SMS_CONFIG['SERVICE_PROVIDER'].get(x),
                            SMS_CONFIG['PRIORITY_SERVICE_PROVIDER']))) != len(SMS_CONFIG['PRIORITY_SERVICE_PROVIDER']):
        raise AZSettingDoesNotExist(
            'SMS configuration: please check `PRIORITY_SERVICE_PROVIDER` and `SERVICE_PROVIDER`')

    for sp_name in SMS_CONFIG['SERVICE_PROVIDER']:
        sp = SMS_CONFIG['SERVICE_PROVIDER'][sp_name]
        if not sp.get('CLASS', None):
            raise AZSettingDoesNotExist(
                f'SMS configuration: please add `CLASS` on `{sp_name}`')

        for r in sp.get('ROUTING', []):
            r['countries'] = list(filter(None, [x.strip().upper() for x in r.get('countries', '').split(',')]))
            r['continents'] = list(filter(None, [x.strip().upper() for x in r.get('continents', '').split(',')]))



import abc

import six

from azmessaging import default_settings as settings
from azmessaging.readers.smsconfig import SMSConfig
from azmessaging.utils import import_class, get_continent


@six.add_metaclass(abc.ABCMeta)
class Reader:

    @abc.abstractmethod
    def get_sms_config(self, identifier) -> SMSConfig:
        pass

    def klass(self, channel: str, identifier: str) -> dict:
        return import_class(settings.CHANNEL_CLASS[channel.upper()])

    def get_sms_sender(self, identifier: str, country_code: str) -> str:
        config = self.get_sms_config(identifier)
        continent = get_continent(country_code).upper()
        for sp_name in config.priorities_service_provider:
            sp = config.service_providers[sp_name]
            for r in sp['ROUTING']:
                if continent in r['continents'] or country_code in r['countries']:
                    kwargs = self._sms_constructor_parameter(config, sp_name)
                    r_copy = r.copy()
                    del r_copy['continents']
                    del r_copy['countries']
                    kwargs.update(r_copy)
                    klass = import_class(sp['CLASS'])
                    return klass(**kwargs)

        # Default
        sp_name = config.default_service_provider
        sp = config.service_providers[sp_name]
        kwargs = self._sms_constructor_parameter(config, sp_name)
        klass = import_class(sp['CLASS'])
        return klass(**kwargs)

    @classmethod
    def _sms_constructor_parameter(cls, config, service_provider_name) -> dict:
        kwargs = {}
        sp = config.service_providers[service_provider_name]
        kwargs.update(sp)
        del kwargs['ROUTING']
        del kwargs['CLASS']
        return kwargs



import pytz


class SMSConfig:
    def __init__(self, default, priorities, service_providers, white_list_countries, black_list_countries):
        self.default_service_provider = default
        self.priorities_service_provider = priorities
        self.service_providers = service_providers
        """
        white list
        """
        if white_list_countries == '__all__':
            wlc = set(pytz.country_names)
        elif white_list_countries == '__none__' or not white_list_countries:
            wlc = set()
        else:
            wlc = set(filter(None, [x.strip().upper() for x in white_list_countries.split(',')]))
        """
        black list
        """
        if black_list_countries == '__all__':
            blc = set(pytz.country_names)
        elif black_list_countries == '__none__' or not black_list_countries:
            blc = set()
        else:
            blc = set(filter(None, [x.strip().upper() for x in black_list_countries.split(',')]))

        self.white_list_countries = wlc - blc

    default_service_provider: str
    priorities_service_provider: [str]
    service_providers: dict
    white_list_countries: [str]
